@@ -3,15 +3,16 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /* exported gCalendarEventTreeClicked, unifinderDoubleClick, unifinderKeyPress,
- *          focusSearch, toggleUnifinder
+ *          focusSearch, ensureUnifinderLoaded, toggleUnifinder
  */
 
-/* import-globals-from ../../../mail/base/content/utilityOverlay.js */
 /* import-globals-from ../../resources/content/mouseoverPreviews.js */
 /* import-globals-from ../src/calFilter.js */
-/* import-globals-from calendar-common-sets.js */
+/* import-globals-from calendar-command-controller.js */
 /* import-globals-from calendar-views-utils.js */
 
+/* globals goToggleToolbar */
+
 /**
  * U N I F I N D E R
  *
@@ -22,7 +23,7 @@
  * window.
  */
 
-var { cal } = ChromeUtils.import("resource://calendar/modules/calUtils.jsm");
+var { cal } = ChromeUtils.import("resource:///modules/calendar/calUtils.jsm");
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 
 // Set this to true when the calendar event tree is clicked to allow for
@@ -40,7 +41,11 @@ var gUnifinderNeedsRefresh = true;
  * @return      Returns true if the unifinder is hidden.
  */
 function isUnifinderHidden() {
-  return document.getElementById("bottom-events-box").hidden;
+  let tabmail = document.getElementById("tabmail");
+  return (
+    tabmail.currentTabInfo.mode.type != "calendar" ||
+    document.getElementById("bottom-events-box").hidden
+  );
 }
 
 /**
@@ -63,24 +68,26 @@ var unifinderObserver = {
   QueryInterface: cal.generateQI([Ci.calICompositeObserver, Ci.nsIObserver, Ci.calIObserver]),
 
   // calIObserver:
-  onStartBatch: function() {},
-
-  onEndBatch: function() {
-    refreshEventTree();
+  onStartBatch() {
+    gUnifinderNeedsRefresh = true;
   },
 
-  onLoad: function() {
-    if (isUnifinderHidden() && !gUnifinderNeedsRefresh) {
+  onEndBatch() {
+    if (isUnifinderHidden()) {
       // If the unifinder is hidden, all further item operations might
       // produce invalid entries in the unifinder. From now on, ignore
       // those operations and refresh as soon as the unifinder is shown
       // again.
       gUnifinderNeedsRefresh = true;
       unifinderTreeView.clearItems();
+    } else {
+      refreshEventTree();
     }
   },
 
-  onAddItem: function(aItem) {
+  onLoad() {},
+
+  onAddItem(aItem) {
     if (
       cal.item.isEvent(aItem) &&
       !gUnifinderNeedsRefresh &&
@@ -90,20 +97,20 @@ var unifinderObserver = {
     }
   },
 
-  onModifyItem: function(aNewItem, aOldItem) {
+  onModifyItem(aNewItem, aOldItem) {
     this.onDeleteItem(aOldItem);
     this.onAddItem(aNewItem);
   },
 
-  onDeleteItem: function(aDeletedItem) {
+  onDeleteItem(aDeletedItem) {
     if (cal.item.isEvent(aDeletedItem) && !gUnifinderNeedsRefresh) {
       this.removeItemFromTree(aDeletedItem);
     }
   },
 
-  onError: function(aCalendar, aErrNo, aMessage) {},
+  onError(aCalendar, aErrNo, aMessage) {},
 
-  onPropertyChanged: function(aCalendar, aName, aValue, aOldValue) {
+  onPropertyChanged(aCalendar, aName, aValue, aOldValue) {
     switch (aName) {
       case "disabled":
         refreshEventTree();
@@ -111,24 +118,28 @@ var unifinderObserver = {
     }
   },
 
-  onPropertyDeleting: function(aCalendar, aName) {
+  onPropertyDeleting(aCalendar, aName) {
     this.onPropertyChanged(aCalendar, aName, null, null);
   },
 
   // calICompositeObserver:
-  onCalendarAdded: function(aAddedCalendar) {
+  onCalendarAdded(aAddedCalendar) {
     if (!aAddedCalendar.getProperty("disabled")) {
-      addItemsFromCalendar(aAddedCalendar, addItemsFromSingleCalendarInternal);
+      if (isUnifinderHidden()) {
+        gUnifinderNeedsRefresh = true;
+      } else {
+        addItemsFromCalendar(aAddedCalendar, addItemsFromSingleCalendarInternal);
+      }
     }
   },
 
-  onCalendarRemoved: function(aDeletedCalendar) {
+  onCalendarRemoved(aDeletedCalendar) {
     if (!aDeletedCalendar.getProperty("disabled")) {
       deleteItemsFromCalendar(aDeletedCalendar);
     }
   },
 
-  onDefaultCalendarChanged: function(aNewDefaultCalendar) {},
+  onDefaultCalendarChanged(aNewDefaultCalendar) {},
 
   /**
    * Add an unifinder item to the tree. It is safe to call these for any
@@ -137,12 +148,12 @@ var unifinderObserver = {
    *
    * @return aItem        The item to add to the tree.
    */
-  addItemToTree: function(aItem) {
+  addItemToTree(aItem) {
     let items;
     let filter = unifinderTreeView.mFilter;
 
     if (filter.startDate && filter.endDate) {
-      items = aItem.getOccurrencesBetween(filter.startDate, filter.endDate, {});
+      items = aItem.getOccurrencesBetween(filter.startDate, filter.endDate);
     } else {
       items = [aItem];
     }
@@ -156,11 +167,11 @@ var unifinderObserver = {
    *
    * @return aItem        The item to remove from the tree.
    */
-  removeItemFromTree: function(aItem) {
+  removeItemFromTree(aItem) {
     let items;
     let filter = unifinderTreeView.mFilter;
     if (filter.startDate && filter.endDate && aItem.parentItem == aItem) {
-      items = aItem.getOccurrencesBetween(filter.startDate, filter.endDate, {});
+      items = aItem.getOccurrencesBetween(filter.startDate, filter.endDate);
     } else {
       items = [aItem];
     }
@@ -168,7 +179,7 @@ var unifinderObserver = {
     unifinderTreeView.removeItems(items.filter(filter.isItemInFilters, filter));
   },
 
-  observe: function() {
+  observe() {
     refreshEventTree();
   },
 };
@@ -178,59 +189,55 @@ var unifinderObserver = {
  * used to add observers, event listeners, etc.
  */
 function prepareCalendarUnifinder() {
-  // Only load once
-  window.removeEventListener("load", prepareCalendarUnifinder);
   let unifinderTree = document.getElementById("unifinder-search-results-tree");
+  // Check that this is not the hidden window, which has no UI elements
+  if (!unifinderTree) {
+    return;
+  }
 
   // Add pref observer
   Services.prefs.addObserver("calendar.date.format", unifinderObserver);
   Services.obs.addObserver(unifinderObserver, "defaultTimezoneChanged");
 
-  // Check if this is not the hidden window, which has no UI elements
-  if (unifinderTree) {
-    // set up our calendar event observer
-    let ccalendar = cal.view.getCompositeCalendar(window);
-    ccalendar.addObserver(unifinderObserver);
+  // set up our calendar event observer
+  let ccalendar = cal.view.getCompositeCalendar(window);
+  ccalendar.addObserver(unifinderObserver);
 
-    // Set up the filter
-    unifinderTreeView.mFilter = new calFilter();
+  // Set up the filter
+  unifinderTreeView.mFilter = new calFilter();
 
-    // Set up the unifinder views.
-    unifinderTreeView.treeElement = unifinderTree;
-    unifinderTree.view = unifinderTreeView;
+  // Set up the unifinder views.
+  unifinderTreeView.treeElement = unifinderTree;
+  unifinderTree.view = unifinderTreeView;
 
-    // Listen for changes in the selected day, so we can update if need be
-    let viewDeck = getViewDeck();
-    viewDeck.addEventListener("dayselect", unifinderDaySelect);
-    viewDeck.addEventListener("itemselect", unifinderItemSelect, true);
+  // Listen for changes in the selected day, so we can update if need be
+  let viewDeck = getViewDeck();
+  viewDeck.addEventListener("dayselect", unifinderDaySelect);
+  viewDeck.addEventListener("itemselect", unifinderItemSelect, true);
 
-    // Set up sortDirection and sortActive, in case it persisted
-    let sorted = unifinderTree.getAttribute("sort-active");
-    let sortDirection = unifinderTree.getAttribute("sort-direction");
-    if (!sortDirection || sortDirection == "undefined") {
-      sortDirection = "ascending";
-    }
-    let tree = document.getElementById("unifinder-search-results-tree");
-    let treecols = tree.getElementsByTagName("treecol");
-    for (let i = 0; i < treecols.length; i++) {
-      let col = treecols[i];
-      let content = col.getAttribute("itemproperty");
-      if (sorted && sorted.length > 0) {
-        if (sorted == content) {
-          unifinderTreeView.sortDirection = sortDirection;
-          unifinderTreeView.selectedColumn = col;
-        }
+  // Set up sortDirection and sortActive, in case it persisted
+  let sorted = unifinderTree.getAttribute("sort-active");
+  let sortDirection = unifinderTree.getAttribute("sort-direction");
+  if (!sortDirection || sortDirection == "undefined") {
+    sortDirection = "ascending";
+  }
+  let treecols = unifinderTree.getElementsByTagName("treecol");
+  for (let col of treecols) {
+    let content = col.getAttribute("itemproperty");
+    if (sorted && sorted.length > 0) {
+      if (sorted == content) {
+        unifinderTreeView.sortDirection = sortDirection;
+        unifinderTreeView.selectedColumn = col;
       }
     }
+  }
 
-    unifinderTreeView.ready = true;
+  unifinderTreeView.ready = true;
 
-    // Display something upon first load. onLoad doesn't work properly for
-    // observers
-    if (!isUnifinderHidden()) {
-      gUnifinderNeedsRefresh = false;
-      refreshEventTree();
-    }
+  // Display something upon first load. onLoad doesn't work properly for
+  // observers
+  if (!isUnifinderHidden()) {
+    refreshEventTree();
   }
 }
 
@@ -288,7 +295,7 @@ function unifinderItemSelect(aEvent) {
  * @return              The passed date's formatted in the default timezone.
  */
 function formatUnifinderEventDateTime(aDatetime) {
-  return cal.getDateFormatter().formatDateTime(aDatetime.getInTimezone(cal.dtz.defaultTimezone));
+  return cal.dtz.formatter.formatDateTime(aDatetime.getInTimezone(cal.dtz.defaultTimezone));
 }
 
 /**
@@ -350,7 +357,7 @@ function unifinderSelect(event) {
 
   // Set up the selected items in the view. Pass in true, so we don't end
   // up in a circular loop
-  currentView().setSelectedItems(selectedItems.length, selectedItems, true);
+  currentView().setSelectedItems(selectedItems, true);
   currentView().centerSelectedItems();
   calendarController.onSelectionChanged({ detail: selectedItems });
   document.getElementById("unifinder-search-results-tree").focus();
@@ -387,10 +394,10 @@ var unifinderTreeView = {
   // Provide a default tree that holds all the functions used here to avoid
   // cludgy if (this.tree) { this.tree.rowCountChanged(...); } constructs.
   tree: {
-    rowCountChanged: function() {},
-    beginUpdateBatch: function() {},
-    endUpdateBatch: function() {},
-    invalidate: function() {},
+    rowCountChanged() {},
+    beginUpdateBatch() {},
+    endUpdateBatch() {},
+    invalidate() {},
   },
 
   ready: false,
@@ -437,18 +444,48 @@ var unifinderTreeView = {
    * Add an item to the unifinder tree.
    *
    * @param aItemArray        An array of items to add.
-   * @param aDontSort         If true, the items will only be appended.
    */
-  addItems: function(aItemArray, aDontSort) {
-    this.eventArray = this.eventArray.concat(aItemArray);
-    let newCount = this.eventArray.length - aItemArray.length - 1;
-    this.tree.rowCountChanged(newCount, aItemArray.length);
+  addItems(aItemArray) {
+    this.tree.beginUpdateBatch();
 
-    if (aDontSort) {
-      this.calculateIndexMap();
+    let bulkSort = aItemArray.length > this.eventArray.length;
+    if (bulkSort || !this.selectedColumn) {
+      // If there's more items being added than already exist,
+      // just append them and sort the whole list afterwards.
+      // If there's no selected column, don't sort at all.
+      let index = this.eventArray.length;
+      this.eventArray = this.eventArray.concat(aItemArray);
+      if (bulkSort && this.selectedColumn) {
+        this.sortItems();
+      } else {
+        this.tree.rowCountChanged(index, aItemArray.length);
+      }
     } else {
-      this.sortItems();
+      // Otherwise, for each item to be added, work out its
+      // new position in the list and splice it in there.
+      // This saves a lot of function calls and calculation.
+      let modifier = this.sortDirection == "descending" ? -1 : 1;
+      let sortKey = unifinderTreeView.selectedColumn.getAttribute("itemproperty");
+      let comparer = cal.unifinder.sortEntryComparer(sortKey);
+
+      let values = this.eventArray.map(item => cal.unifinder.getItemSortKey(item, sortKey));
+      for (let item of aItemArray) {
+        let itemValue = cal.unifinder.getItemSortKey(item, sortKey);
+        let index = values.findIndex(value => comparer(value, itemValue, modifier) >= 0);
+        if (index < 0) {
+          this.eventArray.push(item);
+          this.tree.rowCountChanged(values.length, 1);
+          values.push(itemValue);
+        } else {
+          this.eventArray.splice(index, 0, item);
+          this.tree.rowCountChanged(index, 1);
+          values.splice(index, 0, itemValue);
+        }
+      }
     }
+
+    this.tree.endUpdateBatch();
+    this.calculateIndexMap(true);
   },
 
   /**
@@ -456,7 +493,7 @@ var unifinderTreeView = {
    *
    * @param aItemArray        An array of items to remove.
    */
-  removeItems: function(aItemArray) {
+  removeItems(aItemArray) {
     let indexesToRemove = [];
     // Removing items is a bit tricky. Our getItemRow function takes the
     // index from a cached map, so removing an item from the array will
@@ -495,7 +532,7 @@ var unifinderTreeView = {
   /**
    * Clear all items from the unifinder.
    */
-  clearItems: function() {
+  clearItems() {
     let oldCount = this.eventArray.length;
     this.eventArray = [];
     this.tree.rowCountChanged(0, -oldCount);
@@ -506,16 +543,11 @@ var unifinderTreeView = {
    * Sets the items that should be in the unifinder. This removes all items
    * that were previously in the unifinder.
    */
-  setItems: function(aItemArray, aDontSort) {
+  setItems(aItemArray) {
     let oldCount = this.eventArray.length;
     this.eventArray = aItemArray.slice(0);
     this.tree.rowCountChanged(oldCount - 1, this.eventArray.length - oldCount);
-
-    if (aDontSort) {
-      this.calculateIndexMap();
-    } else {
-      this.sortItems();
-    }
+    this.sortItems();
   },
 
   /**
@@ -527,7 +559,7 @@ var unifinderTreeView = {
    *                                you correctly issued rowCountChanged
    *                                notices.
    */
-  calculateIndexMap: function(aDontInvalidate) {
+  calculateIndexMap(aDontInvalidate) {
     this.eventIndexMap = {};
     for (let i = 0; i < this.eventArray.length; i++) {
       this.eventIndexMap[this.eventArray[i].hashId] = i;
@@ -541,7 +573,7 @@ var unifinderTreeView = {
   /**
    * Sort the items in the unifinder by the currently selected column.
    */
-  sortItems: function() {
+  sortItems() {
     if (this.selectedColumn) {
       let modifier = this.sortDirection == "descending" ? -1 : 1;
       let sortKey = unifinderTreeView.selectedColumn.getAttribute("itemproperty");
@@ -557,7 +589,7 @@ var unifinderTreeView = {
    * @param item      The item to search for.
    * @return          The row index of the passed item.
    */
-  getItemRow: function(item) {
+  getItemRow(item) {
     if (this.eventIndexMap[item.hashId] === undefined) {
       return -1;
     }
@@ -570,7 +602,7 @@ var unifinderTreeView = {
    * @param item      The row index to get the item for.
    * @return          The item at the given row.
    */
-  getItemAt: function(aRow) {
+  getItemAt(aRow) {
     return this.eventArray[aRow];
   },
 
@@ -580,7 +612,7 @@ var unifinderTreeView = {
    * @param event     The DOM mouse event to get the item for.
    * @return          The item under the mouse position.
    */
-  getItemFromEvent: function(event) {
+  getItemFromEvent(event) {
     let row = this.tree.getRowAt(event.clientX, event.clientY);
 
     if (row > -1) {
@@ -594,7 +626,7 @@ var unifinderTreeView = {
    *
    * @param aItemArray        An array of items to select.
    */
-  setSelectedItems: function(aItemArray) {
+  setSelectedItems(aItemArray) {
     if (
       this.doingSelection ||
       !this.tree ||
@@ -607,7 +639,7 @@ var unifinderTreeView = {
     this.doingSelection = true;
 
     // If no items were passed, get the selected items from the view.
-    aItemArray = aItemArray || currentView().getSelectedItems({});
+    aItemArray = aItemArray || currentView().getSelectedItems();
 
     calendarUpdateDeleteCommand(aItemArray);
 
@@ -646,7 +678,7 @@ var unifinderTreeView = {
    * Due to a selection issue described in bug 168211 this method is needed to
    * re-add the selection listeners selection listeners.
    */
-  resetAllowSelection: function() {
+  resetAllowSelection() {
     if (!this.tree) {
       return;
     }
@@ -672,12 +704,12 @@ var unifinderTreeView = {
   // an itemTreeView that these tree views can inherit, that contains this
   // code, and possibly other code related to sorting and storing items. See
   // bug 432582 for more details.
-  getCellProperties: function(aRow, aCol) {
+  getCellProperties(aRow, aCol) {
     let rowProps = this.getRowProperties(aRow);
     let colProps = this.getColumnProperties(aCol);
     return rowProps + (rowProps && colProps ? " " : "") + colProps;
   },
-  getRowProperties: function(aRow) {
+  getRowProperties(aRow) {
     let properties = [];
     let item = this.eventArray[aRow];
     if (item.priority > 0 && item.priority < 5) {
@@ -695,62 +727,62 @@ var unifinderTreeView = {
     }
 
     // Alarm status atom
-    if (item.getAlarms({}).length) {
+    if (item.getAlarms().length) {
       properties.push("alarm");
     }
 
     // Task categories
-    properties = properties.concat(item.getCategories({}).map(cal.view.formatStringForCSSRule));
+    properties = properties.concat(item.getCategories().map(cal.view.formatStringForCSSRule));
 
     return properties.join(" ");
   },
-  getColumnProperties: function(aCol) {
+  getColumnProperties(aCol) {
     return "";
   },
 
-  isContainer: function() {
+  isContainer() {
     return false;
   },
 
-  isContainerOpen: function(aRow) {
+  isContainerOpen(aRow) {
     return false;
   },
 
-  isContainerEmpty: function(aRow) {
+  isContainerEmpty(aRow) {
     return false;
   },
 
-  isSeparator: function(aRow) {
+  isSeparator(aRow) {
     return false;
   },
 
-  isSorted: function(aRow) {
+  isSorted(aRow) {
     return false;
   },
 
-  canDrop: function(aRow, aOrientation) {
+  canDrop(aRow, aOrientation) {
     return false;
   },
 
-  drop: function(aRow, aOrientation) {},
+  drop(aRow, aOrientation) {},
 
-  getParentIndex: function(aRow) {
+  getParentIndex(aRow) {
     return -1;
   },
 
-  hasNextSibling: function(aRow, aAfterIndex) {},
+  hasNextSibling(aRow, aAfterIndex) {},
 
-  getLevel: function(aRow) {
+  getLevel(aRow) {
     return 0;
   },
 
-  getImageSrc: function(aRow, aOrientation) {},
+  getImageSrc(aRow, aOrientation) {},
 
-  getCellValue: function(aRow, aCol) {
+  getCellValue(aRow, aCol) {
     return null;
   },
 
-  getCellText: function(row, column) {
+  getCellText(row, column) {
     let calendarEvent = this.eventArray[row];
 
     switch (column.element.getAttribute("itemproperty")) {
@@ -771,7 +803,7 @@ var unifinderTreeView = {
         return formatUnifinderEventDateTime(eventEndDate);
       }
       case "categories": {
-        return calendarEvent.getCategories({}).join(", ");
+        return calendarEvent.getCategories().join(", ");
       }
       case "location": {
         return calendarEvent.getProperty("LOCATION");
@@ -788,13 +820,13 @@ var unifinderTreeView = {
     }
   },
 
-  setTree: function(tree) {
+  setTree(tree) {
     this.tree = tree;
   },
 
-  toggleOpenState: function(aRow) {},
+  toggleOpenState(aRow) {},
 
-  cycleHeader: function(col) {
+  cycleHeader(col) {
     if (!this.selectedColumn) {
       this.sortDirection = "ascending";
     } else if (!this.sortDirection || this.sortDirection == "descending") {
@@ -806,18 +838,12 @@ var unifinderTreeView = {
     this.sortItems();
   },
 
-  isEditable: function(aRow, aCol) {
+  isEditable(aRow, aCol) {
     return false;
   },
 
-  setCellValue: function(aRow, aCol, aValue) {},
-  setCellText: function(aRow, aCol, aValue) {},
-
-  performAction: function(aAction) {},
-
-  performActionOnRow: function(aAction, aRow) {},
-
-  performActionOnCell: function(aAction, aRow, aCol) {},
+  setCellValue(aRow, aCol, aValue) {},
+  setCellText(aRow, aCol, aValue) {},
 
   outParameter: {}, // used to obtain dates during sort
 };
@@ -840,6 +866,8 @@ function refreshEventTree() {
     cal.view.getCompositeCalendar(window),
     addItemsFromCompositeCalendarInternal
   );
+
+  gUnifinderNeedsRefresh = false;
 }
 
 /**
@@ -887,14 +915,14 @@ function addItemsFromCalendar(aCalendar, aAddItemsInternalFunc) {
     QueryInterface: ChromeUtils.generateQI([Ci.calIOperationListener]),
     mEventArray: [],
 
-    onOperationComplete: function(aOpCalendar, aStatus, aOperationType, aId, aDateTime) {
+    onOperationComplete(aOpCalendar, aStatus, aOperationType, aId, aDateTime) {
       let refreshTreeInternalFunc = function() {
         aAddItemsInternalFunc(refreshListener.mEventArray);
       };
       setTimeout(refreshTreeInternalFunc, 0);
     },
 
-    onGetResult: function(aOpCalendar, aStatus, aItemType, aDetail, aCount, aItems) {
+    onGetResult(aOpCalendar, aStatus, aItemType, aDetail, aItems) {
       refreshListener.mEventArray = refreshListener.mEventArray.concat(aItems);
     },
   };
@@ -936,6 +964,16 @@ function focusSearch() {
   document.getElementById("unifinder-search-field").focus();
 }
 
+/**
+ * The unifinder is hidden if the calendar tab is not selected. When the tab
+ * is selected, this function is called so that unifinder setup completes.
+ */
+function ensureUnifinderLoaded() {
+  if (!isUnifinderHidden() && gUnifinderNeedsRefresh) {
+    refreshEventTree();
+  }
+}
+
 /**
  * Toggles the hidden state of the unifinder.
  */
@@ -949,7 +987,6 @@ function toggleUnifinder() {
   // When the unifinder is hidden, refreshEventTree is not called. Make sure
   // the event tree is refreshed now.
   if (!isUnifinderHidden() && gUnifinderNeedsRefresh) {
-    gUnifinderNeedsRefresh = false;
     refreshEventTree();
   }
 
@@ -960,5 +997,5 @@ function toggleUnifinder() {
   unifinderTreeView.setSelectedItems();
 }
 
-window.addEventListener("load", prepareCalendarUnifinder);
+window.addEventListener("load", prepareCalendarUnifinder, { once: true });
 window.addEventListener("unload", finishCalendarUnifinder);
