@@ -5,18 +5,17 @@
 
 #include "nsParserUtils.h"
 #include "mozilla/NullPrincipal.h"
+#include "mozilla/UniquePtr.h"
 #include "mozilla/dom/DocumentFragment.h"
 #include "mozilla/dom/Element.h"
 #include "mozilla/dom/ScriptLoader.h"
 #include "nsAttrName.h"
-#include "nsAutoPtr.h"
 #include "nsCOMPtr.h"
 #include "nsContentCID.h"
 #include "nsContentUtils.h"
 #include "nsEscape.h"
 #include "nsHTMLParts.h"
 #include "nsHtml5Module.h"
-#include "nsIComponentManager.h"
 #include "nsIContent.h"
 #include "nsIContentSink.h"
 #include "nsIDTD.h"
@@ -24,7 +23,6 @@
 #include "nsIDocumentEncoder.h"
 #include "nsIFragmentContentSink.h"
 #include "nsIParser.h"
-#include "nsISupportsPrimitives.h"
 #include "nsNetCID.h"
 #include "nsNetUtil.h"
 #include "nsParserCIID.h"
@@ -103,15 +101,13 @@ nsParserUtils::ParseFragment(const nsAString& aFragment, uint32_t aFlags,
     // XHTML
     tagStack.AppendElement(NS_LITERAL_STRING(XHTML_DIV_TAG));
     rv = nsContentUtils::ParseFragmentXML(aFragment, document, tagStack, true,
-                                          getter_AddRefs(fragment));
+                                          aFlags, getter_AddRefs(fragment));
   } else {
-    fragment = new DocumentFragment(document->NodeInfoManager());
+    fragment = new (document->NodeInfoManager())
+        DocumentFragment(document->NodeInfoManager());
     rv = nsContentUtils::ParseFragmentHTML(aFragment, fragment, nsGkAtoms::body,
-                                           kNameSpaceID_XHTML, false, true);
-  }
-  if (fragment) {
-    nsTreeSanitizer sanitizer(aFlags);
-    sanitizer.Sanitize(fragment);
+                                           kNameSpaceID_XHTML, false, true,
+                                           aFlags);
   }
 
   if (scripts_enabled) {
